function [W mu_opt kernel_opt runtime] = yada_learn(X, a, b, choiceModel, d_h, mu, kernel, GradOn, numFolds)
    % 
    % [W mu_opt kernel_opt] = yada_learn(X, a, b, choiceModel, d_h, mu,kernel, GradOn, numFolds)
    %
    % Learn a mapping from raw physical space to perceptual space with XAB experiment data and specific choice model.
    %
    % ******************************************************************************************************************
    %  Input parameters:
    % ****************************************************************************************************************** 
    % X: a d_r-by-n matrix, the coordinates of objects in raw space. Each column is an object;
    % a: a n-by-n lower triangular matrix, a_ij is the incorrect number of responses in XAB experiment for object i and j.  
    %    i and j are the indices of the objects in X;
    % b: a n-by-n lower triangular matrix, b_ij is the correct number of responses in XAB experiment for object i and j.  
    %    i and j are the indices of the objects in X;
    % choideModel: a function handle to choice model which specifies the confusion probability = \psi(d). If possible, 
    %              please provide gradient information regarding W as well, which will speed up learning process.
    % d_h: the dimension of perceptual space;
    % mu: the wight of regularizer. If more than one mu is provided, cross validation is used to select one of them.
    % kernel: kernel parameters, which is a structure. Currently, only support RBF kernel, so kernel.type ='rbf' and kernel.sigma 
    %          is the bandwidth of RBF kernel. If more than one sigma is provided, cross validation is used to select one of them.
    % GradOn: If gradient information is provided in choice model function, please set it as 'on', which will speed up the learning.
    %         The default value is 'off'
    % numFolds: The number of folds in cross validation to tune the parameters. The default value is 5.
    % ******************************************************************************************************************
    %  Output parameters:
    % ****************************************************************************************************************** 
    % W: the weight vector learned by yada
    % mu_opt: if more than one mu is provided, mu_opt is the one selected by cross validation. Otherwise, it is the same as input 
    %         parameter mu.
    % kernel_opt: if more than one sigma is provided, kernel_opt is the one selected by cross validation. Otherwise it is the same 
    %             as input parameter kernel. Users should save the output, because yada_map and yada_inverse_map require it as input.
    % runtime: the running time for learning mapping, excluding the time tuning parameters
    %
    
    params.d_h = d_h;
    
    if (nargin > 5)
        params.mus = mu;
    else
        params.mus = 10.^(-7:2);
    end
    
    if (nargin > 6)
        params.kernel = kernel;
        params.kernel.sigmas = kernel.sigma;
    else
        params.kernel.type = 'rbf';
        params.kernel.sigmas = 2.^(-2:0.5:3);
    end
    
    if (nargin > 7)
        params.GradOn = GradOn;
    else
        params.GradOn = 'off';
    end
    
    if (nargin > 8)
        params.numFolds = numFolds;
    else
        params.numFolds = 5;
    end
        
    % Normalize the X
    n = size(X, 2);
    X = (X - repmat(mean(X, 2), 1, n)) ./ repmat(std(X, 0, 2), 1, n);
    
    if ( length(params.mus) == 1 && length(params.kernel.sigmas) == 1 )
        % Only one mu and sigma is specified, learn mapping
        params.mu = mu;
        params.kernel.sigma = params.kernel.sigmas;
        mu_opt = params.mus;
        kernel_opt.type = 'rbf';
        kernel_opt.sigma = params.kernel.sigma;
        [W runtime] = learnMapping(X, a, b, choiceModel, params);
    else
        % using cross validation to tune parameters. 
        [W mu_opt kernel_opt runtime] = learnMapping_tune(X, a, b, choiceModel, params);
    end
        
        
        
        
        
